package com.landray.kmss.sys.attachment.service.spring;

import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.List;
import java.util.Collections;
import java.util.regex.Pattern;
import java.util.ArrayList;
import com.landray.kmss.util.StringUtil;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

public class SysAttachmentHtmlHandler
{
    private Logger logger;
    
    public SysAttachmentHtmlHandler() {
        this.logger = LoggerFactory.getLogger(SysAttachmentHtmlHandler.class);
    }
    
    public String handlingHTMLAgain(final String html) {
        return this.handlingHTMLAgainZoom(html, "0.7");
    }
    
    public String handlingHTMLAgainZoom(String html, final String zoom) {
        if (html == null || "".equals(html)) {
            this.logger.warn("html内容为空");
            return html;
        }
        html = this.handlingCKEditor(html);
        if (StringUtil.isNotNull(zoom)) {
            final float zoomVal = Float.parseFloat(zoom);
            html = this.handlingZoom(html, zoomVal);
        }
        return html;
    }
    
    private String handlingZoom(String html, final float percent) {
        final List<Replace> list = new ArrayList<Replace>();
        StringBuilder sb = new StringBuilder(html);
        Pattern p = Pattern.compile("([1-9]\\d*\\.\\d*px|0\\.\\d*[1-9]\\d0*px| [1-9]\\d*px|'[1-9]\\d*px|\"[1-9]\\d*px|=[1-9]\\d*px|:[1-9]\\d*px)");
        Matcher m = p.matcher(html);
        while (m.find()) {
            final String group = m.group();
            String prefix = "";
            String num = group.replace("px", "");
            int prelength = num.length();
            num = num.replace(" ", "");
            if (prelength != num.length()) {
                prefix = " ";
                prelength = num.length();
            }
            num = num.replace("'", "");
            if (prelength != num.length()) {
                prefix = "'";
                prelength = num.length();
            }
            num = num.replace("\"", "");
            if (prelength != num.length()) {
                prefix = "\"";
                prelength = num.length();
            }
            num = num.replace("=", "");
            if (prelength != num.length()) {
                prefix = "=";
                prelength = num.length();
            }
            num = num.replace(":", "");
            if (prelength != num.length()) {
                prefix = ":";
                prelength = num.length();
            }
            if (!"".equals(num)) {
                final float fnum = Float.valueOf(num);
                if (fnum <= 1.0f) {
                    continue;
                }
                final float newnum = percent * fnum;
                list.add(new Replace(m.start(), m.end(), String.valueOf(prefix) + newnum + "px"));
            }
        }
        p = Pattern.compile("height=\"[0-9]+\"");
        m = p.matcher(html);
        while (m.find()) {
            final String group = m.group();
            final String num2 = group.replace("height=\"", "").replace("\"", "");
            if (!"".equals(num2)) {
                final float fnum2 = Float.valueOf(num2);
                if (fnum2 <= 1.0f) {
                    continue;
                }
                final float newnum2 = percent * fnum2;
                list.add(new Replace(m.start(), m.end(), "height=\"" + newnum2 + "\""));
            }
        }
        p = Pattern.compile("width=\"[0-9]+\"");
        m = p.matcher(html);
        while (m.find()) {
            final String group = m.group();
            final String num2 = group.replace("width=\"", "").replace("\"", "");
            if (!"".equals(num2)) {
                final float fnum2 = Float.valueOf(num2);
                if (fnum2 <= 1.0f) {
                    continue;
                }
                final float newnum2 = percent * fnum2;
                list.add(new Replace(m.start(), m.end(), "width=\"" + newnum2 + "\""));
            }
        }
        Collections.sort(list);
        Collections.reverse(list);
        for (final Replace r : list) {
            sb = sb.replace(r.start, r.end, r.newChar);
        }
        html = sb.toString();
        return html;
    }
    
    private String handlingCKEditor(String html) {
        StringBuilder sb = new StringBuilder(html);
        final Pattern p = Pattern.compile("<div id='_____rtf_____[\\s\\S]*?' style='display:none'>");
        final Matcher m = p.matcher(html);
        if (m.find()) {
            final String group = m.group();
            final String newStr = group.replace("display:none", "");
            sb = sb.replace(m.start(), m.end(), newStr);
            html = sb.toString();
            return this.handlingCKEditor(html);
        }
        return html;
    }
    
    private static void replace(final StringBuilder jsp, final String oldStr, final String newStr) {
        if (jsp == null || jsp.length() == 0 || oldStr == null || newStr == null) {
            return;
        }
        int loopindex;
        for (int index = jsp.indexOf(oldStr); index != -1; index = index + newStr.length() + loopindex) {
            jsp.replace(index, index + oldStr.length(), newStr);
            loopindex = jsp.substring().indexOf(oldStr);
            if (loopindex == -1) {
                index = -1;
                break;
            }
        }
    }
    
    private class Replace implements Comparable<Replace>
    {
        int start;
        int end;
        String newChar;
        
        public Replace(final int start, final int end, final String newChar) {
            this.start = start;
            this.end = end;
            this.newChar = newChar;
        }
        
        @Override
        public int compareTo(final Replace o) {
            return (this.end < o.end) ? -1 : ((this.end == o.end) ? 0 : 1);
        }
    }
}
